@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"runtime"
 	"sort"
 	"sync"
 
@@ -110,16 +111,16 @@ func (c *CompressionMode) Type() string {
 // New returns a new repository with backend be.
 func New(be restic.Backend, opts Options) (*Repository, error) {
 	if opts.Compression == CompressionInvalid {
-		return nil, errors.Fatalf("invalid compression mode")
+		return nil, errors.New("invalid compression mode")
 	}
 
 	if opts.PackSize == 0 {
 		opts.PackSize = DefaultPackSize
 	}
 	if opts.PackSize > MaxPackSize {
-		return nil, errors.Fatalf("pack size larger than limit of %v MiB", MaxPackSize/1024/1024)
+		return nil, fmt.Errorf("pack size larger than limit of %v MiB", MaxPackSize/1024/1024)
 	} else if opts.PackSize < MinPackSize {
-		return nil, errors.Fatalf("pack size smaller than minimum of %v MiB", MinPackSize/1024/1024)
+		return nil, fmt.Errorf("pack size smaller than minimum of %v MiB", MinPackSize/1024/1024)
 	}
 
 	repo := &Repository{
@@ -170,14 +171,8 @@ func (r *Repository) SetDryRun() {
 	r.be = dryrun.New(r.be)
 }
 
-// LoadUnpacked loads and decrypts the file with the given type and ID, using
-// the supplied buffer (which must be empty). If the buffer is nil, a new
-// buffer will be allocated and returned.
-func (r *Repository) LoadUnpacked(ctx context.Context, t restic.FileType, id restic.ID, buf []byte) ([]byte, error) {
-	if len(buf) != 0 {
-		panic("buf is not empty")
-	}
-
+// LoadUnpacked loads and decrypts the file with the given type and ID.
+func (r *Repository) LoadUnpacked(ctx context.Context, t restic.FileType, id restic.ID) ([]byte, error) {
 	debug.Log("load %v with id %v", t, id)
 
 	if t == restic.ConfigFile {
@@ -189,15 +184,17 @@ func (r *Repository) LoadUnpacked(ctx context.Context, t restic.FileType, id res
 	h := restic.Handle{Type: t, Name: id.String()}
 	retriedInvalidData := false
 	var dataErr error
+	wr := new(bytes.Buffer)
+
 	err := r.be.Load(ctx, h, 0, 0, func(rd io.Reader) error {
 		// make sure this call is idempotent, in case an error occurs
-		wr := bytes.NewBuffer(buf[:0])
+		wr.Reset()
 		_, cerr := io.Copy(wr, rd)
 		if cerr != nil {
 			return cerr
 		}
-		buf = wr.Bytes()
 
+		buf := wr.Bytes()
 		if t != restic.ConfigFile && !restic.Hash(buf).Equal(id) {
 			debug.Log("retry loading broken blob %v", h)
 			if !retriedInvalidData {
@@ -221,6 +218,7 @@ func (r *Repository) LoadUnpacked(ctx context.Context, t restic.FileType, id res
 		return nil, err
 	}
 
+	buf := wr.Bytes()
 	nonce, ciphertext := buf[:r.key.NonceSize()], buf[r.key.NonceSize():]
 	plaintext, err := r.key.Open(ciphertext[:0], nonce, ciphertext, nil)
 	if err != nil {
@@ -596,7 +594,7 @@ func (r *Repository) LoadIndex(ctx context.Context) error {
 	})
 
 	if err != nil {
-		return errors.Fatal(err.Error())
+		return err
 	}
 
 	err = r.idx.MergeFinalIndexes()
@@ -604,6 +602,9 @@ func (r *Repository) LoadIndex(ctx context.Context) error {
 		return err
 	}
 
+	// Trigger GC to reset garbage collection threshold
+	runtime.GC()
+
 	if r.cfg.Version < 2 {
 		// sanity check
 		ctx, cancel := context.WithCancel(ctx)
@@ -616,7 +617,7 @@ func (r *Repository) LoadIndex(ctx context.Context) error {
 			}
 		})
 		if invalidIndex {
-			return errors.Fatal("index uses feature not supported by repository version 1")
+			return errors.New("index uses feature not supported by repository version 1")
 		}
 	}
 
@@ -681,7 +682,7 @@ func (r *Repository) CreateIndexFromPacks(ctx context.Context, packsize map[rest
 
 	err = wg.Wait()
 	if err != nil {
-		return invalid, errors.Fatal(err.Error())
+		return invalid, err
 	}
 
 	return invalid, nil
@@ -726,9 +727,9 @@ func (r *Repository) SearchKey(ctx context.Context, password string, maxKeys int
 	r.keyID = key.ID()
 	cfg, err := restic.LoadConfig(ctx, r)
 	if err == crypto.ErrUnauthenticated {
-		return errors.Fatalf("config or key %v is damaged: %v", key.ID(), err)
+		return fmt.Errorf("config or key %v is damaged: %w", key.ID(), err)
 	} else if err != nil {
-		return errors.Fatalf("config cannot be loaded: %v", err)
+		return fmt.Errorf("config cannot be loaded: %w", err)
 	}
 
 	r.setConfig(cfg)
